#' @name BASiCS_Chain
#' @aliases BASiCS_Chain-class
#' 
#' @title The BASiCS_Chain class
#' 
#' @description Container of an MCMC sample of the BASiCS' model parameters 
#' (see Vallejos et al, 2015) as generated by the 
#' function \code{\link[BASiCS]{BASiCS_MCMC}}. 
#' 
#' @slot parameters List of matrices containing MCMC chains for each model parameter. 
#' \describe{
#' \item{mu}{MCMC chain for gene-specific mean expression parameters 
#' \eqn{\mu_i}, biological genes only 
#' (matrix with \code{q.bio} columns, all elements must be positive numbers)}
#' \item{delta}{MCMC chain for gene-specific biological over-dispersion 
#' parameters \eqn{\delta_i}, biological genes only 
#' (matrix with \code{q.bio} columns, all elements must be positive numbers)}
#' \item{phi}{MCMC chain for cell-specific mRNA content normalisation parameters 
#' \eqn{\phi_j} (matrix with \code{n} columns, all elements must be positive 
#' numbers and the sum of its elements must be equal to \code{n})}
#' \item{s}{MCMC chain for cell-specific technical normalisation parameters 
#' \eqn{s_j} (matrix with \code{n} columns, 
#' all elements must be positive numbers)}
#' \item{nu}{MCMC chain for cell-specific random effects \eqn{\nu_j}
#' (matrix with \code{n} columns, all elements must be positive numbers)}
#' \item{theta}{MCMC chain for technical over-dispersion parameter(s) 
#' \eqn{\theta} (matrix, all elements must be positive, 
#' each colum represents 1 batch)}
#' }
#'   
#' @examples
#' 
#' # A BASiCS_Chain object created by the BASiCS_MCMC function.
#' Data <- makeExampleBASiCS_Data()
#' Chain <- BASiCS_MCMC(Data, N = 100, Thin = 2, Burn = 2)
#' 
#' @author Catalina A. Vallejos \email{cnvallej@@uc.cl}
#' 
#' @references 
#' 
#' Vallejos, Marioni and Richardson (2015). PLoS Computational Biology.
#' 
setClass("BASiCS_Chain", 
         representation = representation(
           parameters = "list"), 
         contains = "Versioned",
         validity = function(object) 
         {
            errors <- character()
            
            if(!("parameters" %in% methods::slotNames(object)))
            {
              errors <- "'object' was generated by an old version of BASiCS"
              return(errors)
            }
            else
            {
              if (length(object@parameters$mu) == 0 | length(object@parameters$delta) == 0 | 
                  length(object@parameters$phi) == 0 | length(object@parameters$s) == 0 | 
                  length(object@parameters$nu) ==  0 | length(object@parameters$theta) == 0) 
              {
                errors <- c(errors, "One or more slots are missing")
                return(errors)
              }
              
              N <- nrow(object@parameters$mu)
              n <- ncol(object@parameters$phi)
              if (nrow(object@parameters$delta) != N | nrow(object@parameters$phi) != N | 
                  nrow(object@parameters$s) != N | nrow(object@parameters$nu) != N | 
                  nrow(object@parameters$theta) != N | 
                  ncol(object@parameters$mu) != ncol(object@parameters$delta) | 
                  ncol(object@parameters$s) != n | ncol(object@parameters$nu) != n) 
              {
                errors <- c(errors, "Slots' dimensions are not compatible")
              }
              
              if (sum(!is.finite(object@parameters$mu)) + sum(!is.finite(object@parameters$delta)) + 
                  sum(!is.finite(object@parameters$phi)) + sum(!is.finite(object@parameters$s)) + 
                  sum(!is.finite(object@parameters$nu)) + sum(!is.finite(object@parameters$theta))) 
              {
                errors <- c(errors, "Some slots contains NAs or Infinite values")
              }
              
              if (length(errors) == 0) 
                TRUE else errors  
            }
            
      },
      prototype = prototype(new("Versioned",
                                versions = c("BASiCS_Chain" = 
                                              utils::packageVersion("BASiCS"))))
      )


#' @name BASiCS_Summary
#' @aliases BASiCS_Summary-class
#' 
#' @title The BASiCS_Summary class
#' 
#' @slot parameters List of parameters in which each entry contains a matrix: 
#' first column contains posterior medians, second column contains the 
#' lower limits of an high posterior density interval and third column 
#' contains the upper limits of high posterior density intervals. 
#' \describe{
#' \item{mu}{Posterior medians (1st column), lower (2nd column) and upper 
#' (3rd column) limits of gene-specific mean expression parameters \eqn{\mu_i}.}
#' \item{delta}{Posterior medians (1st column), lower (2nd column) and upper 
#' (3rd column) limits of gene-specific biological over-dispersion parameters 
#' \eqn{\delta_i}, biological genes only }
#' \item{phi}{Posterior medians (1st column), lower (2nd column) and upper 
#' (3rd column) limits of cell-specific mRNA content normalisation 
#' parameters \eqn{\phi_j}}
#' \item{s}{Posterior medians (1st column), lower (2nd column) and upper 
#' (3rd column) limits of cell-specific technical normalisation 
#' parameters \eqn{s[j]}}
#' \item{nu}{Posterior medians (1st column), lower (2nd column) and upper 
#' (3rd column) limits of cell-specific random effects \eqn{\nu_j}}
#' \item{theta}{Posterior median (1st column), lower (2nd column) and upper 
#' (3rd column) limits of technical over-dispersion parameter(s) \eqn{\theta} 
#' (each row represents one batch)}
#' }
#' 
#' @examples
#' 
#' # A BASiCS_Summary object created by the Summary method.
#' Data = makeExampleBASiCS_Data()
#' Chain <- BASiCS_MCMC(Data, N = 100, Thin = 2, Burn = 2)
#' ChainSummary <- Summary(Chain)
#' 
#' @description Container of a summary of a \code{\linkS4class{BASiCS_Chain}} 
#' object. In each slot, first column contains posterior medians; second and 
#' third columns respectively contain the lower and upper limits of an high 
#' posterior density interval (for a given probability).
#' 
setClass("BASiCS_Summary", 
         representation = representation(
           parameters = "list"
         ), 
         contains="Versioned",
         validity = function(object) 
        {
          if (sum(!is.finite(object@parameters$mu)) > 0 | 
              sum(!is.finite(object@parameters$delta)) > 0 | 
              sum(!is.finite(object@parameters$phi)) > 0 | 
              sum(!is.finite(object@parameters$s)) > 0 | 
              sum(!is.finite(object@parameters$nu)) > 0 | 
              sum(!is.finite(object@parameters$theta)) > 0) 
                return("Invalid slots") else { TRUE }
      },
      prototype = prototype(new("Versioned",
                                versions = c("BASiCS_Summary" = 
                                               utils::packageVersion("BASiCS"))))
)
